@@ -14,17 +14,22 @@ use crate::Point2d;
 /// The starting index and size is given by a rectangle,
 /// i.e. x- and y-index don't need to start at zero.
 #[derive(Clone, Debug, Eq, PartialEq, Hash)]
-pub struct Array2d<S: Integer, T> {
+pub struct Array2d<S: Integer, T>
+where
+    usize: TryFrom<S>,
+{
     bounds: BBox2d<S>,
     data: Vec<T>,
 }
 
-impl<S: Integer, T> Array2d<S, T> {
+impl<S: Integer, T> Array2d<S, T>
+where
+    usize: TryFrom<S>,
+{
     /// Creates a new array with the given bounds
     /// that is filled with copies of a given element.
     pub fn new(bounds: BBox2d<S>, d: T) -> Array2d<S, T>
     where
-        usize: TryFrom<S>,
         T: Clone,
     {
         let data = repeat(d).take(bounds.area() as usize).collect::<Vec<_>>();
@@ -36,7 +41,6 @@ impl<S: Integer, T> Array2d<S, T> {
     pub fn new_with<F>(bounds: BBox2d<S>, f: F) -> Array2d<S, T>
     where
         F: Fn(Point2d<S>) -> T,
-        usize: TryFrom<S>,
     {
         let data = bounds.iter().map(f).collect::<Vec<_>>();
         Array2d { bounds, data }
@@ -48,7 +52,6 @@ impl<S: Integer, T> Array2d<S, T> {
     where
         T: Copy,
         S: TryFrom<usize>,
-        usize: TryFrom<S>,
     {
         let y_len = S::try_from(v.len()).unwrap_or(0.into());
         let x_len = if y_len == 0.into() {
@@ -66,7 +69,27 @@ impl<S: Integer, T> Array2d<S, T> {
 
     /// Returns the bounds of the array.
     pub fn bounds(&self) -> BBox2d<S> {
-        self.bounds.clone()
+        self.bounds
+    }
+
+    /// Returns a reference to the element at the index
+    /// or None if the index is out of bounds.
+    pub fn get(&self, index: Point2d<S>) -> Option<&T> {
+        if self.bounds().contains(index) {
+            Some(&self.data[self.bounds.seq_index(index)])
+        } else {
+            None
+        }
+    }
+
+    /// Returns a reference to the element at the index
+    /// or None if the index is out of bounds.
+    pub fn get_mut(&mut self, index: Point2d<S>) -> Option<&mut T> {
+        if self.bounds().contains(index) {
+            Some(&mut self.data[self.bounds.seq_index(index)])
+        } else {
+            None
+        }
     }
 }
 
@@ -96,12 +119,52 @@ mod tests {
     use crate::p2d;
     use crate::Array2d;
 
+    #[test]
+    fn test_from_vec() {
+        let v = vec![vec!['0', '1'], vec!['2', '3']];
+        let a = Array2d::from_vec(v);
+        assert_eq!(a[p2d(0, 0)], '0');
+        assert_eq!(a[p2d(1, 0)], '1');
+        assert_eq!(a[p2d(0, 1)], '2');
+        assert_eq!(a[p2d(1, 1)], '3');
+    }
+
+    #[test]
+    fn test_get() {
+        let r = bb2d(-2..3, -1..2);
+        let a = Array2d::new_with(r, |p| if p == p2d(0, 0) { '*' } else { '.' });
+        assert_eq!(a.get(p2d(0, 0)), Some(&'*'));
+        assert_eq!(a.get(p2d(1, 1)), Some(&'.'));
+        assert_eq!(a.get(p2d(3, 2)), None);
+    }
+
+    #[test]
+    fn test_get_mut() {
+        let r = bb2d(-2..3, -1..2);
+        let mut a = Array2d::new(r, '.');
+        *a.get_mut(p2d(0, 0)).unwrap() = '*';
+        assert_eq!(a[p2d(0, 0)], '*');
+        assert_eq!(a.get(p2d(1, 1)), Some(&'.'));
+        assert_eq!(a.get_mut(p2d(3, 2)), None);
+    }
+
     #[test]
     fn test_index() {
         let r = bb2d(-2..3, -1..2);
-        let mut a = Array2d::new_with(r, |p| if p == p2d(0, 0) { '*' } else { '.' });
+        let a = Array2d::new_with(r, |p| if p == p2d(0, 0) { '*' } else { '.' });
         assert_eq!(a[p2d(0, 0)], '*');
-        a[p2d(1, 1)] = '+';
-        assert_eq!(a[p2d(1, 1)], '+');
+        assert_eq!(a[p2d(1, 1)], '.');
+    }
+
+    #[test]
+    fn test_index_mut() {
+        let r = bb2d(-2..3, -1..2);
+        let mut a = Array2d::new(r, '.');
+        a[p2d(0, 0)] = '*';
+        assert_eq!(a[p2d(0, 0)], '*');
+        assert_eq!(a[p2d(-2, -1)], '.');
+        assert_eq!(a[p2d(-2, 1)], '.');
+        assert_eq!(a[p2d(2, -1)], '.');
+        assert_eq!(a[p2d(2, 1)], '.');
     }
 }
